//
// Copyright (C) 2015-2020 Virgil Security Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     (1) Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//
//     (2) Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in
//     the documentation and/or other materials provided with the
//     distribution.
//
//     (3) Neither the name of the copyright holder nor the names of its
//     contributors may be used to endorse or promote products derived from
//     this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// Lead Maintainer: Virgil Security Inc. <support@virgilsecurity.com>
//

import Foundation

/// Class representing Virgil Card Signature
@objc(VSSCardSignature) public final class CardSignature: NSObject {
    /// Identifier of signer
    /// - Important: Must be unique. Reserved values:
    ///   - Self signatures: "self"
    ///   - Virgil Service signatures: "virgil"
    @objc public let signer: String
    /// Signature data
    @objc public let signature: Data
    /// Additional data
    @objc public let snapshot: Data?
    /// Dictionary with additional data
    @objc public let extraFields: [String: String]?

    /// Initializer
    ///
    /// - Parameters:
    ///   - signer: identifier of signer
    ///   - signature: signature data
    ///   - snapshot: additional data
    ///   - extraFields: dictionary with additional data
    @objc public init(signer: String, signature: Data, snapshot: Data?, extraFields: [String: String]? = nil) {
        self.signer = signer
        self.signature = signature
        self.snapshot = snapshot
        self.extraFields = extraFields

        super.init()
    }
}
